#!/usr/bin/env python
import sys
import os
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import numpy as np
from csv import DictReader
from copy import copy
from collections import defaultdict
import sqlite3

# For main plots
LINE_STYLE = ['k-.', 'b--o' , 'c:^', 'r--s'  ]#, 'b:o'   , 'c:^'    , 'r--s'   , 'r:s'    ]
schednames = ['uedf', 'os_iso', 'prior', 'no_io']
legendnames= ['UEDF', 'OS-Isolated', r'Prior MC$^2$', 'No-I/O']


# For LBA plots
#LINE_STYLE = ['c--^'   , 'c:^'    , 'r--s'   , 'r:s'    , 'm--D'       , 'm:D'        ]
#schednames = ['CBA-PST', 'CBA-RST', 'WBA-PST', 'WBA-RST', 'CBA-PST-OPT', 'CBA-RST-OPT']
#legendnames= ['CBA/PST', 'CBA/RST', 'WBA/PST', 'WBA/RST', 'LBA/PST'    , 'LBA/RST'    ]

"""
util_dist	= ["util_Amod_Bmod_Clight","util_Amod_Bmod_Cmod","util_Alight_Blight_Cheavy"]
task_util	= ['lightweight','modweight','heavyweight']
pet_sens	= ['sens_regular','sens_way']
memory_util	= ['wss-heavy','wss-light']
period		= ['period_long','period_short','period_mod']
net_dist	= ['net_few', 'net_many', 'net_mod','net_none']
has_cam		= ['has_cam','no_cam']
"""
util_dist	= ['util_Alight_Blight_Cheavy']
task_util	= ['heavyweight']
pet_sens	= ['sens_regular']
memory_util	= ['wss-heavy']
period		= ['period_long']
net_dist	= ['net_few']
has_cam		= ['no_cam']

cats = ['util_dist','task_util','pet_sens','memory_util','period','net_dist','has_cam']

def quot(in_str):
	return "'" + in_str + "'"

plot_space = [ (ud, tu, ps, mu, p, nd, hc) \
	for ud in util_dist\
	for tu in task_util\
	for ps in pet_sens\
	for mu in memory_util\
	for p in period\
	for nd in net_dist\
	for hc in has_cam\
	if not (nd is 'net_none' and hc is 'no_cam')]



def dp2title(dp):
    return '--'.join(dp)

def main():
    db = "../sql/results_refined.db"
    outdir = "../"
    total_time = 0
    total_count = 0
    fnum = 0
    conn = sqlite3.connect(db)
    curses = conn.cursor()
    for plot_dp in plot_space:

	#print plot_dp
	#print cats
	#print ' AND '.join(['='.join(kv) for kv in zip(cats, map(quot,plot_dp))])
	curses.execute("SELECT sys_util, uedf, os_iso, prior, no_io FROM plot \
		WHERE " + ' AND '.join(['='.join(kv) for kv in zip(cats,map(quot,plot_dp))]) +\
		" ORDER BY sys_util")

	npdata=dict()
	npdata['uedf'] = []
	npdata['os_iso'] = []
	npdata['prior'] = []
	npdata['no_io'] = []
	npdata['sys_util'] = []
	for row in curses.fetchall():
	    npdata['sys_util'].append(row[0])
	    npdata['uedf'].append(row[1])
	    npdata['os_iso'].append(row[2])
	    npdata['prior'].append(row[3])
	    npdata['no_io'].append(row[4])

        plt.figure(figsize=(8,4))
        plt.ylabel("Schedulability")
        plt.xlabel("System Utilization")
        lw = 4.0
	schedset = []
	#if 'no_cam' in filename and 'net_none' in filename:
	#    LINE_STYLE = ['k-.', 'b--o']#, 'b:o'   , 'c:^'    , 'r--s'   , 'r:s'    ]
	#    schednames = ['EDF', 'Managed']
	#    legendnames= ['UEDF', 'OS-Isolated']
	#else:
	#    LINE_STYLE = ['k-.', 'b--o' , 'c:^'  ]#, 'b:o'   , 'c:^'    , 'r--s'   , 'r:s'    ]
	#    schednames = ['EDF', 'Managed', 'Unmanaged']
	#    legendnames= ['UEDF', 'OS-Isolated', r'Prior MC$^2$']
    
        for ndx, sched in enumerate(schednames):
            plt.plot(npdata['sys_util'], npdata[sched], LINE_STYLE[ndx], label=legendnames[ndx], linewidth=lw, markersize = lw*3)
            plt.xlim(0,9)
            lw -= 0.3
        
	for idx in range(len(npdata['sys_util'])):
	    if npdata['prior'][idx] > 0.3 and npdata['prior'][idx] < 0.7\
		and npdata['sys_util'][idx] >= 5.0 and npdata['sys_util'][idx] <= 7:
		plt.plot(npdata['sys_util'][idx], npdata['prior'][idx], marker="o", markerfacecolor='none',markeredgecolor='red',ms=20.0,mew=2.0)
		print npdata['prior'][idx]
		break; 
        plt.legend(loc="upper right")
        plt.savefig(outdir + "CIRCLE_"+ dp2title(plot_dp) + ".pdf", bbox_inches='tight', pad_inches=0.1)
        plt.savefig(outdir + "CIRCLE_" + dp2title(plot_dp) + ".png", bbox_inches='tight', pad_inches=0.1)

        plt.close()

            
        #for ndx, count in enumerate(npdata['Count']):
        #    if count != None:
        #        total_count += int(count)
        
    #print 'count is ' + str(total_count)    
        
if __name__ == '__main__':
    main()
