#!/usr/bin/env python
import sys
import os
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import numpy as np
from csv import DictReader
from copy import copy
from collections import defaultdict

# For main plots
LINE_STYLE = ['b--o' , 'c:^'  ]#, 'b:o'   , 'c:^'    , 'r--s'   , 'r:s'    ]
schednames = ['Managed', 'Unmanaged']
legendnames= ['OS-Isolated', r'Prior MC$^2$']


# For LBA plots
#LINE_STYLE = ['c--^'   , 'c:^'    , 'r--s'   , 'r:s'    , 'm--D'       , 'm:D'        ]
#schednames = ['CBA-PST', 'CBA-RST', 'WBA-PST', 'WBA-RST', 'CBA-PST-OPT', 'CBA-RST-OPT']
#legendnames= ['CBA/PST', 'CBA/RST', 'WBA/PST', 'WBA/RST', 'LBA/PST'    , 'LBA/RST'    ]


def main():
    datadir = "../results_930_2/"
    outdir = "../plots_930_2_nouedf/"
    total_time = 0
    total_count = 0
    fnum = 0
    #filenames = ['light-light-heavy--harmonic-long--uni-heavy--uni-heavy--uni-medium--regular--uni-large--uni-light--fixed',         'medium-medium-medium--harmonic-long--uni-medium--uni-light--uni-heavy--way-heavy--uni-small--uni-heavy--fixed', 'medium-medium-medium--harmonic-long--uni-medium--uni-light--uni-medium--regular--uni-large--uni-light--fixed']
    #for filename in filenames:
    for filename in os.listdir(os.getcwd() + "/" + datadir):
        d = DictReader(open(datadir + filename, 'r'))
        data = defaultdict(list)
        for row in d:
            for key, value in row.iteritems():
                data[key].append(value)

        npdata = defaultdict(list)
        npdata["sys_util"] = np.asarray(data["sys_util"])
        idx_ord = npdata["sys_util"].argsort()

        for key in data.keys():
            npdata[key] = np.asarray(data[key])[idx_ord]

        plt.figure(figsize=(8,4))
        plt.ylabel("Schedulability")
        plt.xlabel("System Utilization")
        lw = 4.0
        for ndx, sched in enumerate(schednames):
            plt.plot(npdata["sys_util"], npdata[sched], LINE_STYLE[ndx], label=legendnames[ndx], linewidth=lw, markersize = lw*3)
            plt.xlim(0,9)
            lw -= 0.3
        
#         if filename is 'light-light-heavy--harmonic-long--uni-heavy--uni-heavy--uni-medium--regular--uni-large--uni-light--fixed':
#             for ndx in range(len(npdata['N-RST'])):
#                 if float(npdata['N-RST'][ndx]) < 0.7 and float(npdata['N-RST'][ndx]) > 0.3:
#                     print "circling " + npdata["sys_util"][ndx] + ", " + npdata['N-RST'][ndx]
#                     plt.plot([npdata["sys_util"][ndx]], [npdata['N-RST'][ndx]], marker='o', markerfacecolor='none', markeredgecolor='r', ms=20.0, mew = 2.0)
#                     break
# #                     
# #         #         if npdata['util_dist'][0] == 'light-heavy-light' and npdata['task_util'][0] == 'uni-medium':
# #         #              plt.plot([npdata["sys_util"][22]], [npdata['MC2-Iso'][22]], marker='s', markerfacecolor='none', markeredgecolor='g', ms=20.0, mew = 2.0)
# #         # 
# #         #         if npdata['util_dist'][0] == 'light-light-heavy':
# #         #              plt.plot([npdata["sys_util"][35]], [npdata['MC2-Iso'][35]], marker='s', markerfacecolor='none', markeredgecolor='g', ms=20.0, mew = 2.0)


        plt.legend(loc="upper right")
        plt.savefig(outdir + "pdfs/" + filename + ".pdf", bbox_inches='tight', pad_inches=0.1)
        plt.savefig(outdir + "pngs/" + filename + ".png", bbox_inches='tight', pad_inches=0.1)

        plt.close()

            
        for ndx, count in enumerate(npdata['Count']):
#             for scheme in ['N-PST', 'N-RST', 'CBA-PST', 'CBA-RST', 'WBA-PST', 'WBA-RST']:
#                 tm = abs(float(npdata[scheme + '-Time'][ndx]))
#                 if tm < 10:
#                     total_time += tm*float(count)
            if count != None:
                total_count += int(count)
        
        
        
        
#             
#         fnum += 1
#         if fnum > 5:
#             break

#         break
#         fig = plt.figure(figsize = (1.5,1.3))
#         ax = plt.gca()
#         for style, sched in enumerate(legendnames):
#             plt.plot([], [], LINE_STYLE[style], label=sched+' ['+str(style+1)+']', linewidth=2.0)
#     
#         figleg = plt.figure(figsize = (6.5, 0.8))
#         leg = plt.figlegend(*ax.get_legend_handles_labels(), loc = 'upper left', ncol=3)
# 
#         leg.get_frame().set_edgecolor('w')
# 
#         figleg.savefig("../plots_o/" + "sched-legend" + ".pdf")
#     
    print "count is " + str(total_count)
#     print "time is", total_time/(3600*24)

if __name__ == '__main__':
    main()
