#!/usr/bin/env python

from collections import defaultdict

from distributions import *
from schedcat.model.LLCLayout import *
from schedcat.model.LLC_params import *
import schedcat.model.ohc as ohc
import random
import time

from pulp import *
import gurobipy

import numpy as np

def LPSizeBuffers(ts):
    problem = LpProblem("Linear",LpMaximize)
    
    M = LpVariable.matrix("M", range(len(ts.buffers)), lowBound = 0)
    
    B2M = defaultdict()
    
    for b_i, b in enumerate(ts.buffers):
        B2M[b] = M[b_i]
        
    for tsk in ts:
        if tsk.node == None:
            continue
        bsum = LpAffineExpression()
        for buff in tsk.R:
            bsum += B2M[buff]
        for buff in tsk.W:
            bsum += B2M[buff]
            
        problem += bsum <= tsk.msz
    
    problem += lpSum(M)
    
    status = problem.solve(GUROBI(msg=0))

    if LpStatus[status] == 'Optimal':
        for b_i, b in enumerate(ts.buffers):
            if b.R.level == b.W.level:
                b.size_m = int(M[b_i].varValue/2)+1
                b.slots = 2
            else:
                b.size_m = int(M[b_i].varValue/3)+1
                b.slots = 3
    else:
        print "impossibly didn't solve the un-unsolveable"
        
