#!/usr/bin/env python

from collections import defaultdict

from distributions import *
from schedcat.model.LLCLayout import *
from schedcat.model.LLC_params import *
import schedcat.model.ohc as ohc
import random
import time

from pulp import *
import gurobipy

import numpy as np
from math import ceil

# capped_wss = {'B' : [512, 512, 512, 512],
#               'C' : 512}
 
inflationDRAM = 1.48
inflationMemBus = 1.38

def LP_WBA2(ts):
    capped_wss = {'B' : [(0 if len(ts.level('B').core(c)) == 0 else max([t.wss for t in ts.level('B').core(c)])) for c in range(CORES)], 
                  'C' : (0 if len(ts.level('C')) == 0 else max([t.wss for t in ts.level('C')]))}
                  
    AB = area(0, 16)
    C = area(8, 16)
    csch = scheme([AB for i in range(CORES)], [AB for i in range(CORES)], C, 16, 16)
   
    #----------------------- Gather Data -----------------------------#
    
    UA_Adata = np.zeros((CORES, csch.mW+1))
    UA_Bdata = np.zeros((CORES, csch.mW+1))
    UA_Cdata = np.zeros((CORES, csch.mW+1))
    
    UB_Bdata = np.zeros((CORES, csch.mW+1))
    UB_Cdata = np.zeros((CORES, csch.mW+1))
    
    UC_Cdata = np.zeros((csch.mW+1))
    hdata = np.zeros((csch.mW+1))
    Hdata = np.zeros((csch.mW+1))
   
    ts.A = ts.level('A')
    ts.B = ts.level('B')
    ts.C = ts.level('C')
 
    minAperiods = []
    for core in range(0, CORES):
        subts = ts.A.core(core)
        if len(subts) == 0:
            minAperiods.append(0)
        else:
            minAperiods.append(min([t.period for t in subts]))
    
    for w in range(0,csch.mW+1):
	wsch = scheme([area(w, csch.A[i].S) for i in range(0, CORES)],
                      [area(w, csch.B[i].S) for i in range(0, CORES)],
                      area(w, csch.C.S), csch.mW, csch.mS)
              

        ts.setScheme(wsch)
        for core in range(0, CORES):              
            UA_Adata[core][w] = ts.A.core(core).utilization(atlevel = 'A', approximate = True, inflated = True, c_taskset = ts.C, minAperiods = minAperiods, mx_wss = capped_wss, bandwidth = ts.bandwidth)
            UA_Bdata[core][w] = ts.A.core(core).utilization(atlevel = 'B', approximate = True, inflated = True, c_taskset = ts.C, minAperiods = minAperiods, mx_wss = capped_wss, bandwidth = ts.bandwidth)
            UA_Cdata[core][w] = ts.A.core(core).utilization(atlevel = 'C', approximate = True, inflated = True, c_taskset = ts.C, minAperiods = minAperiods, mx_wss = capped_wss, bandwidth = ts.bandwidth)
        
            UB_Bdata[core][w] = ts.B.core(core).utilization(atlevel = 'B', approximate = True, inflated = True, c_taskset = ts.C, minAperiods = minAperiods, mx_wss = capped_wss, bandwidth = ts.bandwidth)
            UB_Cdata[core][w] = ts.B.core(core).utilization(atlevel = 'C', approximate = True, inflated = True, c_taskset = ts.C, minAperiods = minAperiods, mx_wss = capped_wss, bandwidth = ts.bandwidth)
        
        UAi = defaultdict()
        UAi['B'] = [UA_Bdata[core][w] for core in range(0, CORES)]
        UAi['C'] = [UA_Cdata[core][w] for core in range(0, CORES)]
    
        UAi = ts.secondaryInflateConst(UAi, minAperiods = minAperiods)
        for core in range(0, CORES):
            UA_Bdata[core][w] = UAi['B'][core]
            UA_Cdata[core][w] = UAi['C'][core]
     
        UC_Cdata[w] = ts.C.utilization('C', approximate = True, inflated = True, c_taskset = ts.C, minAperiods = minAperiods, mx_wss = capped_wss, bandwidth = ts.bandwidth)
    
        hdata[w] = ts.C.max_utilization('C', approximate = True, inflated = True, c_taskset = ts.C, minAperiods = minAperiods, mx_wss = capped_wss, bandwidth = ts.bandwidth)
        Hdata[w] = ts.C.utilization('C', approximate = True, inflated = True, c_taskset = ts.C, minAperiods = minAperiods, mx_wss = capped_wss, heaviest = CORES-1, bandwidth = ts.bandwidth)
     
     
    #----------------------- Create LP Variables -----------------------------#
    problem = LpProblem("Hybrid",LpMinimize)
    
    UA_A = LpVariable.matrix("UA_A", range(CORES), lowBound = 0)
    UA_B = LpVariable.matrix("UA_B", range(CORES), lowBound = 0)
    UA_C = LpVariable.matrix("UA_C", range(CORES), lowBound = 0)
    
    UB_B = LpVariable.matrix("UB_B", range(CORES), lowBound = 0)
    UB_C = LpVariable.matrix("UB_C", range(CORES), lowBound = 0)
        
    UC_C = LpVariable("UC_C", lowBound = 0)
    
    h = LpVariable("h", lowBound = 0)
    H = LpVariable("H", lowBound = 0)
    
    Wab = LpVariable.matrix("Wab", range(CORES), lowBound = 0, cat='Integer')
    Wc = LpVariable("Wc", lowBound = 0, cat = 'Integer')
    
    #----------------------- Constraint Set 1 -----------------------------#
    Wc.upBound = csch.mW
    for b_j in range(CORES):
        Wab[b_j].upBound = csch.mW
    problem += lpSum(Wab) + Wc <= csch.mW
            
    
    #----------------------- Constraint Set 2 -----------------------------#
    for b_j in range(0, CORES):
        UA_A[b_j].upBound = 1
        UA_B[b_j].upBound = 1
        UA_C[b_j].upBound = 1
    
        UB_B[b_j].upBound = 1
        UB_C[b_j].upBound = 1
        
        if not len(ts.A.core(b_j)) is 0:
            invminp = 1.0/min([t.period for t in ts.A.core(b_j)])
        else:
            invminp = 0
            
        tckinfw = ohc.PTCK/(ohc.PTCK-ohc.ohcvals['Max']['TCK'])
        tckinfa = ohc.PTCK/(ohc.PTCK-ohc.ohcvals['Avg']['TCK'])
        
        w_var = Wab[b_j]
        
        IA_B = ohc.CPMD*w_var*csch.B[i].S*invminp*tckinfw
        IA_C = ohc.CPMDavg*w_var*csch.B[i].S*invminp*tckinfa
        
        
        for w_k in range(0,csch.mW):
            C1 = (UA_Adata[b_j][w_k+1]-UA_Adata[b_j][w_k])
            C2 = UA_Adata[b_j][w_k]-w_k*C1
            problem += (UA_A[b_j]-C1*w_var >= C2)
            C1 = (UA_Bdata[b_j][w_k+1]-UA_Bdata[b_j][w_k])
            C2 = UA_Bdata[b_j][w_k]-w_k*C1
            problem += (UA_B[b_j]-C1*w_var >= C2 + IA_B)
            C1 = (UA_Cdata[b_j][w_k+1]-UA_Cdata[b_j][w_k])
            C2 = UA_Cdata[b_j][w_k]-w_k*C1
            problem += (UA_C[b_j]-C1*w_var >= C2 + IA_C)
        
            C1 = (UB_Bdata[b_j][w_k+1]-UB_Bdata[b_j][w_k])
            C2 = UB_Bdata[b_j][w_k]-w_k*C1
            problem += (UB_B[b_j]-C1*w_var >= C2)
            C1 = (UB_Cdata[b_j][w_k+1]-UB_Cdata[b_j][w_k])
            C2 = UB_Cdata[b_j][w_k]-w_k*C1
            problem += (UB_C[b_j]-C1*w_var >= C2)

    UC_C.upBound = CORES
    for w_k in range(0, csch.mW):
        C1 = (UC_Cdata[w_k+1]-UC_Cdata[w_k])
        C2 = UC_Cdata[w_k]-w_k*C1
        problem += (UC_C-C1*Wc >= C2)
    
    #----------------------- Constraint Set 3 -----------------------------#
        problem += h >= hdata[w_k] + (Wc-w_k)*(hdata[w_k+1]-hdata[w_k])
        problem += H >= Hdata[w_k] + (Wc-w_k)*(Hdata[w_k+1]-Hdata[w_k])
        
    #----------------------- Constraint Set 4 -----------------------------#
   
    INT_MASTER = ohc.ohcvals['Max']['RQC'] + ohc.ohcvals['Max']['ISR']

    INT_MASTER_AVG = ohc.ohcvals['Avg']['RQC'] + ohc.ohcvals['Avg']['ISR']
    C_loss = 0

    #At Level C interrupts are like higher criticality tasks, don't need to inflate on per-task basis
    C_loss = 0
    C_loss += (INT_MASTER_AVG + ohc.ipi_int_avg)*sum(1/t.period for t in ts.C) #Level C release IPI delay
    allIO = []
    for level in ['A','B']:
        for c in range(CORES):
            allIO.extend(ts.ioDict[level][c])
    allIO.extend(ts.ioDict['C'])

    C_loss += sum([ ((ohc.cam_usb_cpy_avg + ohc.cam_int_avg) if tio.device is 'camera' else ohc.ssd_int_avg)/tio.period for tio in allIO]) #Level C interrupt

    for c in range(CORES):
        problem += UA_A[c] <= 1
        problem += UA_B[c] + UB_B[c] <= 1
    
    problem += lpSum(UA_C) + lpSum(UB_C) + UC_C <= CORES-C_loss
        
    eps = 1e-6
    
    problem += lpSum(UA_C) + lpSum(UB_C) + (CORES-1)*h + H <= CORES-eps-C_loss
    
    problem += lpSum(UA_C) + lpSum(UB_C) + UC_C
    
    
    #----------------------- Solve -----------------------------#
    start = time.clock() 
    status = problem.solve(GUROBI(msg=0, timeLimit = 480))
    fin = time.clock()
    
    rntm = -1
    if LpStatus[status] != 'Not Solved':
        rntm = fin-start

    if LpStatus[status] == 'Optimal':
        ps = scheme([area(0, csch.A[i].S) for i in range(0,CORES)],
                    [area(0, csch.B[i].S) for i in range(0,CORES)],
                    area(0, MAX_SETS), MAX_WAYS, MAX_SETS)
        
        ps.A = [val.rounded() for val in ps.A]
        ps.B = [val.rounded() for val in ps.B]
        ps.C = ps.C.rounded()
        
        return ps, rntm
    if LpStatus[status] == 'Not Solved':
        return None, rntm
        
    return None, rntm


def LPLLCNapsack(ts_mcx, olap = True, debug = False):
    #Tired of swapping these around. Fix later by full replacement of one over other (ts vs ts_mcx)
    ts = ts_mcx
    capped_wss = {
	'B': [(0 if len(ts_mcx.level('B').core(c)) == 0 else max([t.wss for t in ts_mcx.level('B').core(c)])) for c in range(CORES)],
	'C': (0 if len(ts_mcx.level('C')) == 0 else max([t.wss for t in ts_mcx.level('C')]))
    }

    LWmax = 4
    Nbin = 16*LWmax

    AB = area(0,16)
    C = area(8,16)

    csch = scheme([AB for i in range(CORES)], [AB for i in range(CORES)], C, 16, 16)
    ts_mcx.A = ts_mcx.level('A')
    ts_mcx.B = ts_mcx.level('B')
    ts_mcx.C = ts_mcx.level('C')

    #----------------------- Gather Data -----------------------------#
    UA_Adata = np.zeros((CORES, csch.mW+1))
    UA_Bdata = np.zeros((CORES, csch.mW+1))
    UA_Cdata = np.zeros((CORES, csch.mW+1))

    UB_Bdata = np.zeros((CORES, csch.mW+1))
    UB_Cdata = np.zeros((CORES, csch.mW+1))
    UC_Cdata = np.zeros(csch.mW+1)
    uC_Cdata = np.zeros((len(ts_mcx.C), csch.mW+1))

    minAperiods = []
    for core in range(0, CORES):
        if len(ts_mcx.A.core(core)) == 0:
            minAperiods.append(0)
        else:
            minAperiods.append(min([t.period for t in ts_mcx.A.core(core)]))

    for w in range(0,csch.mW+1):
        wsch = scheme([area(w, csch.A[i].S) for i in range(0, CORES)],
                      [area(w, csch.B[i].S) for i in range(0, CORES)],
                      area(w, csch.C.S), csch.mW, csch.mS)
        ts_mcx.setScheme(wsch)
        for core in range(0, CORES):
            UA_Adata[core][w] = ts_mcx.A.core(core).utilization('A', approximate = True, inflated = True, c_taskset = ts_mcx.C, minAperiods = minAperiods, mx_wss = capped_wss, bandwidth = ts.bandwidth)
            UA_Bdata[core][w] = ts_mcx.A.core(core).utilization('B', approximate = True, inflated = True, c_taskset = ts_mcx.C, minAperiods = minAperiods, mx_wss = capped_wss, bandwidth = ts.bandwidth)
            UA_Cdata[core][w] = ts_mcx.A.core(core).utilization('C', approximate = True, inflated = True, c_taskset = ts_mcx.C, minAperiods = minAperiods, mx_wss = capped_wss, bandwidth = ts.bandwidth)
            
            UB_Bdata[core][w] = ts_mcx.B.core(core).utilization('B', approximate = True, inflated = True, c_taskset = ts_mcx.C, minAperiods = minAperiods, mx_wss = capped_wss, bandwidth = ts.bandwidth)
            UB_Cdata[core][w] = ts_mcx.B.core(core).utilization('C', approximate = True, inflated = True, c_taskset = ts_mcx.C, minAperiods = minAperiods, mx_wss = capped_wss, bandwidth = ts.bandwidth)
        
        UAi = defaultdict()
        UAi['B'] = [UA_Bdata[core][w] for core in range(0, CORES)]
        UAi['C'] = [UA_Cdata[core][w] for core in range(0, CORES)]
        
        UAi = ts_mcx.secondaryInflateConst(UAi, minAperiods = minAperiods)
        for core in range(0, CORES):
            UA_Bdata[core][w] = UAi['B'][core]
            UA_Cdata[core][w] = UAi['C'][core]
        
        UC_Cdata[w] = ts_mcx.C.utilization('C', approximate = True, inflated = True, c_taskset = ts_mcx.C, minAperiods = minAperiods, mx_wss = capped_wss, bandwidth = ts.bandwidth)
        
        for t_i in range(len(ts_mcx.C)):
            uC_Cdata[t_i][w] = ts_mcx.C[t_i].utilization('C', approximate = True, inflated = True, c_taskset = ts_mcx.C, minAperiod = 0, mx_wss = capped_wss, bandwidth = ts.bandwidth)
    

    #----------------------- Create LP Variables -----------------------------#
    problem = LpProblem("Hybrid",LpMinimize)

    UA_A = LpVariable.matrix("UA_A", range(CORES), lowBound = 0)
    UA_B = LpVariable.matrix("UA_B", range(CORES), lowBound = 0)
    UA_C = LpVariable.matrix("UA_C", range(CORES), lowBound = 0)

    UB_B = LpVariable.matrix("UB_B", range(CORES), lowBound = 0)
    UB_C = LpVariable.matrix("UB_C", range(CORES), lowBound = 0)

    UC_C = LpVariable("UC_C", lowBound = 0)

    uC_C = LpVariable.matrix("uC_C", range(len(ts_mcx.C)), lowBound = 0, cat='Continuous')

    L = LpVariable.matrix("L", range(len(ts_mcx.buffers)), 0, cat='Continuous')

    Wab = LpVariable.matrix("Wab", range(CORES), lowBound=0, cat='Integer')
    Wc = LpVariable("Wc", lowBound=0, cat='Integer')
    Wl = LpVariable("Wl", 0, LWmax, cat='Integer')

    b_h = LpVariable("b_h", lowBound = 0)
    b_H = LpVariable("b_H", lowBound = 0)

    z = LpVariable.matrix("z", range(len(ts_mcx.C)), lowBound = 0)
    Z = LpVariable.matrix("Z", range(len(ts_mcx.C)), lowBound = 0)

    #----------------------- Constraint Set 1 -----------------------------#

    Wc.upBound = csch.mW
    for b_j in range(CORES):
        Wab[b_j].upBound = csch.mW
    problem += lpSum(Wab) + Wc + Wl<= csch.mW

    #----------------------- Constraint Set 2 -----------------------------#
    #for i, c in enumerate(Wa):
    #    if olap:
    #        problem += Wa[i] + Wc + Wl <= csch.mW
    #        problem += Wb[i] + Wc + Wl <= csch.mW
    #    else:
    #        problem += Wa[i] + Wb[i] + Wc + Wl <= csch.mW
    #
    #    problem += WO[i] >= Wa[i] + Wb[i] + Wc + Wl - csch.mW

    #----------------------- Constraint Set 3 -----------------------------#
    for q_i, q in enumerate(ts_mcx.buffers):
        problem += L[q_i] <= q.size_m

    assignment_sum = LpAffineExpression()
    for q_i, q in enumerate(ts_mcx.buffers):
        assignment_sum += L[q_i]*q.slots
    problem += assignment_sum <= 128*16*Wl

    #----------------------- Constraint Set 4 -----------------------------#

    #Stuff needed in multiple places
    tckinfw = ohc.PTCK/(ohc.PTCK-ohc.ohcvals['Max']['TCK'])
    tckinfa = ohc.PTCK/(ohc.PTCK-ohc.ohcvals['Avg']['TCK'])

    #----------------------- Buffer Opt Expressions -----------------------#
    q2i_c = defaultdict()

    for q_i, q in enumerate(ts_mcx.buffers):
        q2i_c[q] = L[q_i]

    Dcache = defaultdict()

    for t_i, tsk in enumerate(ts_mcx):
        Dcache[tsk] = {'A' : LpAffineExpression(),
                       'B' : LpAffineExpression(),
                       'C' : LpAffineExpression()}
        #for buff in tsk.R:
        #    if buff.tech == 'SBP': #copying
        #        Dcache[tsk]['A'] += ts_mcx.a_inflate_choice*q2i_c[buff]*(ohc.LINE_ACC_CBR   -ohc.LINE_ACC_CLCR   )/tsk.period,
        #        Dcache[tsk]['B'] +=                         q2i_c[buff]*(ohc.LINE_ACC_CBR   -ohc.LINE_ACC_CLCR   )/tsk.period,
        #        Dcache[tsk]['C'] +=                         q2i_c[buff]*(ohc.LINE_ACC_CBRavg-ohc.LINE_ACC_CLCRavg)/tsk.period
        #    else: #in-place
        #        Dcache[tsk]['A'] += ts_mcx.a_inflate_choice*(q2i_c[buff]/128.0)*ohc.CPMD   *tckinfw/tsk.period,
        #        Dcache[tsk]['B'] +=                         (q2i_c[buff]/128.0)*ohc.CPMD   *tckinfw/tsk.period,
        #        Dcache[tsk]['C'] +=                         (q2i_c[buff]/128.0)*ohc.CPMDavg*tckinfa/tsk.period

        #for buff in tsk.W:
        #    if buff.tech == 'SBP': #copying
        #        Dcache[tsk]['A'] += ts_mcx.a_inflate_choice*q2i_c[buff]*(ohc.LINE_ACC_CBW   -ohc.LINE_ACC_CLCW   )/tsk.period,
        #        Dcache[tsk]['B'] +=                         q2i_c[buff]*(ohc.LINE_ACC_CBW   -ohc.LINE_ACC_CLCW   )/tsk.period,
        #        Dcache[tsk]['C'] +=                         q2i_c[buff]*(ohc.LINE_ACC_CBWavg-ohc.LINE_ACC_CLCWavg)/tsk.period
        #    else: #in-place
        #        Dcache[tsk]['A'] += ts_mcx.a_inflate_choice*(q2i_c[buff]/128.0)*ohc.CPMD   *tckinfw/tsk.period,
        #        Dcache[tsk]['B'] +=                         (q2i_c[buff]/128.0)*ohc.CPMD   *tckinfw/tsk.period,
        #        Dcache[tsk]['C'] +=                         (q2i_c[buff]/128.0)*ohc.CPMDavg*tckinfa/tsk.period

        for buff in tsk.R:
            Dcache[tsk]['A'] += 1.5*q2i_c[buff]*(ohc.ipcDict['R'][buff.tech][buff.size_m] - ohc.ipcDict['R']['CL'][buff.size_m])/tsk.period/buff.size_m
            Dcache[tsk]['B'] += q2i_c[buff]*(ohc.ipcDict['R'][buff.tech][buff.size_m] - ohc.ipcDict['R']['CL'][buff.size_m])/tsk.period/buff.size_m
            Dcache[tsk]['C'] += q2i_c[buff]*(ohc.ipcDictAvg['R'][buff.tech][buff.size_m] - ohc.ipcDictAvg['R']['CL'][buff.size_m])/tsk.period/buff.size_m

        for buff in tsk.W:
            Dcache[tsk]['A'] += 1.5*q2i_c[buff]*(ohc.ipcDict['W'][buff.tech][buff.size_m] - ohc.ipcDict['W']['CL'][buff.size_m])/tsk.period/buff.size_m
            Dcache[tsk]['B'] += q2i_c[buff]*(ohc.ipcDict['W'][buff.tech][buff.size_m] - ohc.ipcDict['W']['CL'][buff.size_m])/tsk.period/buff.size_m
            Dcache[tsk]['C'] += q2i_c[buff]*(ohc.ipcDictAvg['W'][buff.tech][buff.size_m] - ohc.ipcDictAvg['W']['CL'][buff.size_m])/tsk.period/buff.size_m


    #----------------------- Constraint Set 5 -----------------------------#
    for i in range(0, CORES):
        UA_A[i].upBound = 1
        UA_B[i].upBound = 1
        UA_C[i].upBound = 1

        UB_B[i].upBound = 1
        UB_C[i].upBound = 1

        if not len(ts_mcx.A.core(i)) is 0:
            invminp = 1.0/min([t.period for t in ts_mcx.A.core(i)])
        else:
            invminp = 0

        IA_B = ohc.CPMD*Wab[i]*csch.B[i].S*invminp*tckinfw if olap else 0
        IA_C = ohc.CPMDavg*Wab[i]*csch.B[i].S*invminp*tckinfa if olap else 0

        for j in range(0,csch.mW):
            C1 = (UA_Adata[i][j+1]-UA_Adata[i][j])
            C2 = UA_Adata[i][j]-j*C1
            problem += (UA_A[i]-C1*Wab[i] >= C2)
            C1 = (UA_Bdata[i][j+1]-UA_Bdata[i][j])
            C2 = UA_Bdata[i][j]-j*C1
            problem += (UA_B[i]-C1*Wab[i] >= C2 + IA_B)
            C1 = (UA_Cdata[i][j+1]-UA_Cdata[i][j])
            C2 = UA_Cdata[i][j]-j*C1
            problem += (UA_C[i]-C1*Wab[i] >= C2 + IA_C)

            C1 = (UB_Bdata[i][j+1]-UB_Bdata[i][j])
            C2 = UB_Bdata[i][j]-j*C1
            problem += (UB_B[i]-C1*Wab[i] >= C2)
            C1 = (UB_Cdata[i][j+1]-UB_Cdata[i][j])
            C2 = UB_Cdata[i][j]-j*C1
            problem += (UB_C[i]-C1*Wab[i] >= C2)

    UC_C.upBound = CORES
    for w in range(0, csch.mW):
        C1 = (UC_Cdata[w+1]-UC_Cdata[w])
        C2 = UC_Cdata[w]-w*C1
        problem += (UC_C-C1*Wc >= C2)

    for t_i, tsk in enumerate(ts_mcx.C):
        tailC = - Dcache[tsk]['C']
        for w in range(csch.mW):
            C1 = uC_Cdata[t_i][w+1]-uC_Cdata[t_i][w]
            C2 = uC_Cdata[t_i][w]-w*C1
            problem += uC_C[t_i]-C1*Wc >= C2 + tailC

    #----------------------- Constraint Set 6 -----------------------------#
    h = b_h+lpSum(z)
    H = 3*b_H+lpSum(Z)

    for t_i in range(len(ts_mcx.C)):
        problem += z[t_i] >= uC_C[t_i]-b_h
        problem += Z[t_i] >= uC_C[t_i]-b_H

    #----------------------- Constraint Set 7 -----------------------------#
    INT_MASTER = ohc.ohcvals['Max']['RQC'] + ohc.ohcvals['Max']['ISR']

    INT_MASTER_AVG = ohc.ohcvals['Avg']['RQC'] + ohc.ohcvals['Avg']['ISR']
    C_loss = 0


    #At Level C interrupts are like higher criticality tasks, don't need to inflate on per-task basis
    C_loss = 0
    C_loss += (INT_MASTER_AVG + ohc.ipi_int_avg)*sum(1/t.period for t in ts.C) #Level C release IPI delay
    allIO = []
    for level in ['A','B']:
        for c in range(CORES):
            allIO.extend(ts.ioDict[level][c])
    allIO.extend(ts.ioDict['C'])

    C_loss += sum([ ((ohc.cam_usb_cpy_avg + ohc.cam_int_avg) if tio.device is 'camera' else ohc.ssd_int_avg)/tio.period for tio in allIO]) #Level C interrupt



    DA_C = LpAffineExpression()
    DB_C = LpAffineExpression()
    DA_A = [LpAffineExpression() for p in range(CORES)]
    DA_B = [LpAffineExpression() for p in range(CORES)]
    DA_C = [LpAffineExpression() for p in range(CORES)]
    DB_B = [LpAffineExpression() for p in range(CORES)]
    DB_C = [LpAffineExpression() for p in range(CORES)]
    for c in range(0,CORES):
        for tsk in ts_mcx.A.core(c):
            DA_A[c] += Dcache[tsk]['A']
            DA_B[c] += Dcache[tsk]['B']
            DA_C += Dcache[tsk]['C']
        for tsk in ts_mcx.B.core(c):
            DB_B[c] += Dcache[tsk]['B']
            DB_C += Dcache[tsk]['C']

        problem += UA_A[c] - DA_A[c] <= 1
        problem += UA_B[c] + UB_B[c] - DA_B[c] - DB_B[c] <= 1
    

    DC_C = LpAffineExpression()
    for tsk in ts_mcx.C:
        DC_C += Dcache[tsk]['C']

    problem += lpSum(UA_C) + lpSum(UB_C) - DA_C - DB_C + UC_C - DC_C <= CORES - C_loss
    eps = 1e-6

    problem += lpSum(UA_C) + lpSum(UB_C) - DA_C - DB_C + (CORES-1)*h + H <= CORES-eps - C_loss
    problem += lpSum(UA_C) + lpSum(UB_C) - DA_C - DB_C + UC_C - DC_C

    #----------------------- Solve -----------------------------#
    status = problem.solve(GUROBI(msg=0))

    if LpStatus[status] == 'Optimal':
        ps = scheme([area(0, csch.A[i].S) for i in range(0,CORES)],
                    [area(0, csch.B[i].S) for i in range(0,CORES)],
                    area(0, MAX_SETS), MAX_WAYS, MAX_SETS)

        ps.A = [val.rounded() for val in ps.A]
        ps.B = [val.rounded() for val in ps.B]
        ps.C = ps.C.rounded()

        return ps, 0

    #return None, 0, -1
    return None, 0

   


