#!/usr/bin/env python

from __future__ import division

import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
LINE_STYLE = ['b:+', 'g-', 'r-s', 'c--', 'k-.']

from math   import ceil, floor
import traceback
import argparse
import random
import time
import numpy as np
from collections import defaultdict
from csv import DictWriter
from util.stats import proportion_ci

from multiprocessing import Pool, cpu_count
from schedcat.util.storage import storage
from generator.generator import DesignPointGenerator

from distributions import *

from schedcat.model.LLC_params import *
from schedcat.generator.mcx_tasks import TaskGeneratorMCX
from schedcat.generator.tasks import uniform_int
from LLCOptimizer import *
from LLCCanvas import LLCCanvas
from DAGGenerator import *
import schedcat.model.ohc as ohc
import loadohs
import graph

from sched_check import *

import multiprocessing
# We must import this explicitly, it is not imported by the top-level
# multiprocessing module.
import multiprocessing.pool
from random import randint

from BufferSizer import *

MIN_SAMPLES = 10 #10
MAX_SAMPLES = 20 #300
MAX_CI      = 0.05 #.05
CONFIDENCE  = 0.95 #.95

CORE_BANDWIDTH = 45

topdir = "~/LLC_Code/MCXSchedStudy/"

#Note this is applied per each crit level seperately
def makeDependencies(ts, dp):
    # swap the squeeze task into random position
    shuf = random.randint(0, len(ts)-1)
    ts[-1], ts[shuf] = ts[shuf], ts[-1]

    nrTasks = len(ts)
    subtasksets = []
    count = 0

    while count < len(ts):
        compSize = NAMED_NUM_GRAPHS[dp.num_graphs]()
        subts = ts[count:count+compSize]
        subtasksets.append(subts)
        inc = compSize
        #print inc
        for tsk in ts[count + compSize:count+inc]:
            tsk.node = None
        count += inc
    for tsk in ts:
        tsk.R = []
        tsk.W = []
    graphs = []
    for subts in subtasksets:
        graphsz = len(subts)
        min_depth = max(1, int(NAMED_HEIGHT_FACTORS[dp.depth_factor][0] * graphsz))
        max_depth = max(1, int(NAMED_HEIGHT_FACTORS[dp.depth_factor][1] * graphsz))
        gg = graph.GraphGenerator(tasks.uniform_int(graphsz, graphsz),
                                 tasks.uniform_int(min_depth, max_depth),
                                 NAMED_SHAPES[dp.node_placement],
                                 NAMED_FAN[dp.fan_out], 3,
                                 NAMED_EDGE_HOP[dp.edge_distance],
                                 graph.uniform(1,1), graph.uniform(1,1),
                                 False, False)
        g = gg.graph()

        assert len(g.nodes) == graphsz

        # bind tasks/nodes, and compute node wss
        for i in range(graphsz):
            g.nodes[i].task = subts[i]
            subts[i].node = g.nodes[i]

            for e in g.nodes[i].inEdges:
                e.R = subts[i]
            for e in g.nodes[i].outEdges:
                e.W = subts[i]

            subts[i].R = g.nodes[i].inEdges
            subts[i].W = g.nodes[i].outEdges
            subts[i].graph = g

            for b in subts[i].R:
                b.size_m = NAMED_IPC_SIZES[dp.ipc_size]()
                b.slots = 2
            for b in subts[i].W:
                b.size_m = NAMED_IPC_SIZES[dp.ipc_size]()
                b.slots = 2

        # single root
        root = [r for r in g.nodes if r.isSrc][0]

        subA = [t for t in subts if t.level == 'A']
        subB = [t for t in subts if t.level == 'B']
        subC = [t for t in subts if t.level == 'C']

        if len(subA) > 0:
            aPeriod = subA[0].period
            for t in subA:
                t.adjust_to_new_period(aPeriod)
                t.period = aPeriod
                t.deadline = aPeriod

        if len(subB) > 0:
            bPeriod = subB[0].period
            for t in subB:
                t.adjust_to_new_period(bPeriod)
                t.period = bPeriod
                t.deadline = bPeriod
        if len(subC) > 0:
            cPeriod = subC[0].period
            for t in subC:
                t.adjust_to_new_period(cPeriod)
                t.period = cPeriod
                t.deadline = cPeriod

        graphs.append(g)


    # tag each graph with an id
    for i,g in enumerate(graphs):
        g.id = i

    return ts, graphs, subtasksets



def make_mcx_task_set(dp):
    tgMCX = TaskGeneratorMCX(period = NAMED_PERIODS[dp.period],
                             util = NAMED_UTILIZATIONS[dp.task_util],
                             util_dist = NAMED_UTIL_DISTS[dp.util_dist],
                             mem_util = NAMED_MEMORY[dp.memory_util],
                             avg_percent = AVG_PERCENT,
                             a_inflation = uniform(1.5, 1.5),
                             pfuncs = P)
                             
    ts = tgMCX.make_task_set(max_util = dp.sys_util)
        
    ts.m = CORES
        
    ts.scheme = bypass_scheme()

    bandwidth = NAMED_BANDWIDTHS[dp.bandwidth]()

    thresh = NAMED_COPY_DIST[dp.copy_dist]()

    tsA, graphsA, subtasksetsA = makeDependencies(ts.level('A'), dp)
    tsB, graphsB, subtasksetsB = makeDependencies(ts.level('B'), dp)

    for tsk in ts:
        if tsk.level is 'C':
            tsk.R = []
            tsk.W = []
    ts.subsets = {'A' : subtasksetsA, 'B' : subtasksetsB}

    ts.buffers = []


    for g in graphsA:
        ts.buffers.extend(g.edges)
    for g in graphsB:
        ts.buffers.extend(g.edges)

    #make srcs I/O tasks
    srcs = {
        'A': [tsk for tsk in tsA if tsk.node is not None and tsk.node.isSrc],
        'B': [tsk for tsk in tsB if tsk.node is not None and tsk.node.isSrc],
    }

    ts.ioDict = {
        'A': dict([ (i,[]) for i in range(CORES) ]),
        'B': dict([ (i,[]) for i in range(CORES) ]),
        'C': [] ,
    }

    samplefunc = uniform(0, 1)

    ts.bandwidth = 0
    ts.hasCam = False

    for level in ['A', 'B']:
        levBandwidth = 0;
        src = srcs[level]
        for tsk in src:
            if levBandwidth > bandwidth/3:
                break
            if samplefunc() > thresh:
                tsk.device = 'camera'
                ts.hasCam = True
            else:
                tsk.device = 'ssd'
            levBandwidth = levBandwidth + 0.6/tsk.period*1000*1000 #0.6 is buffer in MB, period is in us, expected bandwidth unit is MB/s
        ts.bandwidth = ts.bandwidth + levBandwidth

    cBandwidth = 0;
    tsC = ts.level('C')
    for tsk in tsC:
        if cBandwidth > bandwidth/3:
            break
        if samplefunc() > thresh:
            tsk.device = 'camera'
            ts.hasCam = True
        else:
            tsk.device = 'ssd'
        cBandwidth = cBandwidth + 0.6/tsk.period*1000*1000
    ts.bandwidth = ts.bandwidth + cBandwidth

    return ts

def ioDictSetup(ts):
    ts.ioDict = {
        'A': dict([ (i,[]) for i in range(CORES) ]),
        'B': dict([ (i,[]) for i in range(CORES) ]),
        'C': [] ,
    }
    for tsk in ts:
        if tsk.device is 'none':
            continue
        #print tsk.device
        if tsk.level is 'C':
            ts.ioDict[tsk.level].append(tsk)
        else:
            ts.ioDict[tsk.level][tsk.core].append(tsk)

def complete(results, n):
    if n < MIN_SAMPLES:
        return False
    elif n > MAX_SAMPLES:
	print 'Samples exceeded'
        return True
    else:
        for key in results:
            if proportion_ci(results[key], n, CONFIDENCE) > MAX_CI:
                return False
        return True

def update_mean2(old_mean, n, new_sample):
    return (old_mean*n + new_sample)/(n+1)

def psForMode(g_i, lpDRAM, usePST):
    C = area(0, 16)
    AB = []
    if usePST:
        AB.append(area(0, lpDRAM[len(lpDRAM)-1]))
        AB.extend([area(0, lpDRAM[g_i][b_j]) for b_j in range(0, 3)])
    else:
        AB = [area(0, lpDRAM[g_i][b_j]) for b_j in range(0, CORES)]
        
    return scheme(AB, AB, C, 16, 16)

def mem_check_managed(ts):
	BANK = 32768 #Get accurate number later
	for i in range(CORES):
		total = 0
		ts_i_A = ts.level('A').core(i)
		ts_i_B = ts.level('B').core(i)
		for tsk in ts_i_A + ts_i_B:
			total += tsk.footprint + ceil(tsk.io_buff/4)
		if total > BANK:
			return False
	ctotal = 0;
	for tsk in ts.level('C'):
		ctotal += tsk.footprint + ceil(tsk.io_buff/4)
	if ctotal > (8 - CORES)*BANK:
		return False
	return True
    
def mem_check_unmanaged(ts):
	#How mem managed here?
	#randomly assign buffers to banks
	return True

def process_dp(dp, debug = False):
    
    time_res = defaultdict(float)
    sched_res = defaultdict(float)
    count = defaultdict(int)

    n = 0

    first = 1
    if debug:
        print "\n", titlefor(dp), "\n"

    while not complete(sched_res, n):
        #print "looping"
        ts = make_mcx_task_set(dp)

        if first and debug:
            print "\nTask system info:"
            print "num tasks:\tA:", len(ts.level('A')), "\tB:", len(ts.level('B')), ",\tC:", len(ts.level('C'))
            print "\n"

            print "wss stats"
            wsses = [ tsk.wss for tsk in ts ]
            print "min:",round(min(wsses), 2), "\tavg:", round(sum(wsses)/(1.0 + len(wsses)), 2), "\tmax:", round(max(wsses),2), "\n"

            print "io info"
            print "num cameras:", len([tsk for tsk in ts if tsk.device is 'camera'])
            print "num ssds:", len([tsk for tsk in ts if tsk.device is 'ssd'])
            print "system bandwidth:", round(ts.bandwidth,2),"\n"

        ts.compute_costs(CORE_BANDWIDTH + ts.bandwidth, False)
        ts.useCshared()

        if debug and first:
            print "example task costs"
            onetask = ts[0]
            print onetask
            onetask.print_costs()
            print "\n"
        

        ts.setLLCAlloc(bypass_scheme())
        ohc.SetOheads(ts, 'mc2') 

    
	#---------------------Prior----------------------#
	
        ts.coreAssign(useMC2 = True)
        ioDictSetup(ts)
	ts.applyNaive()
	ts.applySBP()
    
        ts.setLLCAlloc(bypass_scheme())
	ts.add_buffer_copy_times()
	ts.add_io_copy_times()
        ts.inflateABinterrupts()

	schedPrior = False
	timePrior = time.clock()
        psPrior, filler = LP_WBA2(ts)
        if psPrior is not None:
            	schedPrior = True
        timePrior = time.clock() - timePrior
	

        #------------------Random------------------------#
        ts.compute_costs(CORE_BANDWIDTH + ts.bandwidth, True)
        ts.useCshared()
        ts.add_buffer_copy_times()
        ts.add_io_copy_times()
        ts.inflateABinterrupts()
        for tsk in ts:
            unmnwt = unmanwt()
            for lvl in ['A','B']:
                for s in range(17):
                    for w in range(17):
                        tsk.cost[lvl][s][w] *= unmnwt
        timeRandom = time.clock()
        schedRandom = False
        psRandom, filler = LP_WBA2(ts)
        if psRandom is not None:
            schedRandom = True
        timeRandom = time.clock() - timeRandom

        #If random, which suffers all negative effects with no benefits can schedule, anything can schedule
        if schedRandom:        
            sched_res['Random']		= update_mean2(sched_res['Random'],	n, True)
            sched_res['Prior']		= update_mean2(sched_res['Prior'],	n, True)
            sched_res['IO']		= update_mean2(sched_res['IO'],		n, True)
            sched_res['IPC']		= update_mean2(sched_res['IPC'],	n, True)
	    sched_res['Both']		= update_mean2(sched_res['Both'],	n, True)
        
            time_res['Random-Time']	= update_mean2(time_res['Random-Time'],	n, timeRandom)
            time_res['Prior-Time']	= update_mean2(time_res['Prior-Time'],	n, 0)
            time_res['IO-Time']     	= update_mean2(time_res['IO-Time'], 	n, 0)
            time_res['IPC-Time']     	= update_mean2(time_res['IPC-Time'], 	n, 0)
            time_res['Both-Time']     	= update_mean2(time_res['Both-Time'], 	n, 0)
        
            count['Count'] = n+1

            n+=1
            first = 0
            continue
        


	###-------------IO--------------------###

        ts.compute_costs(CORE_BANDWIDTH + ts.bandwidth, True)
        ts.useCshared()
        ts.add_buffer_copy_times()
        ts.inflateABinterrupts()
	timeIO = time.clock()
	schedIO = False
	psIO, filler = LP_WBA2(ts)
	if psIO is not None:
		schedIO = True
	timeIO = time.clock() - timeIO

	###-------------IPC-No-DAG-------------------###

	#IPC is known to be superior to Prior (WBA2 soln is included in LPLLCNapsack's soln space)
	timeIPCNoDAG = 0
	schedIPCNoDAG = False
	if not schedPrior:
                ts.compute_costs(CORE_BANDWIDTH + ts.bandwidth, False)
                ts.useCshared()
                ts.applyNaive()
                ts.applySBP()
		ts.applyCE()
		ts.add_buffer_copy_times()
                ts.add_io_copy_times()
                ts.inflateABinterrupts()
		timeIPCNoDAG = time.clock()
		psIPCNoDAG, filler = LPLLCNapsack(ts)
		if psIPCNoDAG is not None:
			schedIPCNoDAG = True
		timeIPCNoDAG = time.clock() - timeIPCNoDAG
	else:
		schedIPCNoDAG = True

	###----------------------Both-No-DAG--------------------###

	schedBothNoDAG = False
	timeBothNoDAG = 0
	#Adding IPC can only help the IO case
	if not schedIO:
                ts.compute_costs(CORE_BANDWIDTH + ts.bandwidth, True)
                ts.useCshared()
                ts.applyNaive()
                ts.applySBP()
		ts.applyCE()
		ts.add_buffer_copy_times()
                ts.inflateABinterrupts()
		timeBothNoDAG = time.clock()
		psBothNoDAG, filler = LPLLCNapsack(ts)
		if psBothNoDAG is not None:
			schedBothNoDAG = True
		timeBothNoDAG = time.clock() - timeBothNoDAG
	else:
		schedBothNoDAG = True
		

	###---------------------IPC-DAG----------------------------###
	#DAG version gets False if nonDAG version succeeded

	DAGAssigned = False
	schedIPCDAG = False
	timeIPCDAG = 0
        if not schedIPCNoDAG:
                ts.compute_costs(CORE_BANDWIDTH + ts.bandwidth, False)
                ts.useCshared()
		ts.dagAssign('A')
		ts.dagAssign('B')
		ioDictSetup(ts)
		ts.applyNaive()
		ts.applySBP()
		ts.applyCE()
		ts.add_buffer_copy_times()
                ts.add_io_copy_times()
                ts.inflateABinterrupts()
		DAGAssigned = True
	

		timeIPCDAG = time.clock()
		psIPCDAG, filler = LPLLCNapsack(ts)
		if psIPCDAG is not None:
			schedIPCDAG = True
		timeIPCDAG = time.clock() - timeIPCDAG

	###--------------------------Both-DAG-----------------------------###

	schedBothDAG = False
	timeBothDAG = 0
	#Adding IPC management to IO has no tradeoffs
	if not schedBothNoDAG:
		if not DAGAssigned:
			ts.dagAssign('A')
			ts.dagAssign('B')
			ioDictSetup(ts)
                ts.compute_costs(CORE_BANDWIDTH + ts.bandwidth, True)
                ts.useCshared()
		ts.applyNaive()
		ts.applySBP()
		ts.applyCE()
		ts.add_buffer_copy_times()
                ts.inflateABinterrupts()
                DAGAssigned = True
		
		timeBothDAG = time.clock()
		psBothDAG, filler = LPLLCNapsack(ts)
		if psBothDAG is not None:
			schedBothDAG = True
		timeBothDAG = time.clock() - timeBothDAG



	sched_res['Random']		= update_mean2(sched_res['Random'],	n, schedRandom)
        sched_res['Prior']		= update_mean2(sched_res['Prior'],	n, schedPrior)
        sched_res['IO']			= update_mean2(sched_res['IO'],		n, schedIO)
        sched_res['IPC']		= update_mean2(sched_res['IPC'],	n, schedIPCNoDAG or schedIPCDAG)
	sched_res['Both']		= update_mean2(sched_res['Both'],	n, schedBothNoDAG or schedBothDAG)
        #if schedOpt:
        #	pollute_res['IOCores']		= update_mean2(pollute_res['IOCores'],	pollcount1, IOCores)
        #	pollute_res['polluted']		= update_mean2(pollute_res['polluted'],	pollcount1, polluted)
	#	pollcount1+=1
        #if schedOpt2:
        #	pollute_res['IOCores2']		= update_mean2(pollute_res['IOCores2'],	pollcount2, IOCores2)
        #	pollute_res['polluted2']	= update_mean2(pollute_res['polluted2'],	pollcount2, polluted2)
	#	pollcount2+=1
	#sched_res['DMAIdeal']		= update_mean2(sched_res['DMAIdeal'],	n, schedDMAIdeal)
 
        #sched_res['ALL'] = update_mean2(sched_res['ALL'] , n, schedManaged or schedUnmanaged) No point if managed always > unmanaged
        
        time_res['Random-Time']		= update_mean2(time_res['Random-Time'],	n, timeRandom)
        time_res['Prior-Time']		= update_mean2(time_res['Prior-Time'],		n, timePrior)
        time_res['IO-Time']     	= update_mean2(time_res['IO-Time'], 		n, timeIO)
        time_res['IPC-Time']     	= update_mean2(time_res['IPC-Time'], 		n, timeIPCNoDAG + timeIPCDAG)
        time_res['Both-Time']     	= update_mean2(time_res['Both-Time'], 		n, timeBothNoDAG + timeBothDAG)
        
        count['Count'] = n+1

        n+=1
        first = 0
        
    return dict(dp.items() + sched_res.items() + time_res.items() + count.items())

def myrange(start, end, inc):
    return np.arange(start, end+inc, inc)
    
def valid(dp):
    return True
    
def thread_tests(args, outs, design_points):
    pool = Pool(processes = args.processors)
    try:
        for i, row in enumerate(pool.imap_unordered(process_dp, design_points)):
            if not row:
                continue
#             sys.stderr.write('\rdone {0:%}'.format(i/len(design_points)))
            print 'done {0:%}'.format(i/len(design_points))
#             print '\rdone {0:%}'.format(i/len(design_points))
#             print row
            title = titlefor(row)
            outs[title].writerow(row)
        pool.close()
    except Exception as e:
        print "hit an exception"
        print e
        pool.terminate()
        pool.join()

def sequential_tests(outs, design_points):
    for i, dp in enumerate(design_points):
	row = process_dp(dp, debug = True)
	if not row:
	    continue
	print 'done {0:%}'.format(i/len(design_points))
	title = titlefor(row)
	outs[title].writerow(row)

def titlefor(headers):
    return headers['util_dist'] + "--" + headers['period'] + "--" + headers['task_util'] + "--" + headers['memory_util'] + "--" + headers['bandwidth'] + "--" + headers['ipc_size'] + "--" + headers['copy_dist']

def main():
    outdir = "results_refined"
    parser = argparse.ArgumentParser()
    # parser.add_argument('-d', "--datafile", type = argparse.FileType('w'),
                        # default = sys.stdout,
                        # help = "File to output csv file to")
    parser.add_argument('-p', "--plot", action='store_true',
                        help = "For plotting generated PETs")
    #parser.add_argument('-m', "--processors", default=5, type = int,
    parser.add_argument('-m', "--processors", default=cpu_count(), type = int,
                        help = "Number of processors to execute on")

    parser.add_argument('-s', "--sequential", default=0, type = int, help = "Run some number of design points sequentially")

    parser.add_argument('-r', "--reuse", default = 0, type = int, help = "number of new scenarios to run")

    args = parser.parse_args()
    exp = storage()

    if 1:
        exp.task_util = NAMED_UTILIZATIONS.keys()
        exp.period = NAMED_PERIODS.keys()
        exp.memory_util = NAMED_MEMORY.keys() #mem util is max reload time
        exp.util_dist = NAMED_UTIL_DISTS.keys()
        exp.sys_util = myrange(0.2, 9.0, 0.4)
    
        exp.ipc_size = NAMED_IPC_SIZES.keys()
        exp.bandwidth = NAMED_BANDWIDTHS.keys()
        exp.copy_dist = NAMED_COPY_DIST.keys()

    exp.num_graphs = NAMED_NUM_GRAPHS.keys()
    exp.depth_factor = NAMED_HEIGHT_FACTORS.keys()
    exp.node_placement = NAMED_SHAPES.keys()
    exp.fan_out = NAMED_FAN.keys()
    exp.edge_distance = NAMED_EDGE_HOP.keys()

    design_points_tmp = [dp for dp in DesignPointGenerator(exp, is_valid = valid)]
    if (args.sequential != 0):
	design_points_tmp = design_points_tmp[:args.sequential];
    design_points = []

    random.seed(43215)
    
    loadohs.load("../oheads/")
    
    donealready = []
    if args.reuse:
	donealready = os.listdir("../results_refined/")

    if args.reuse > 0:
        num_left = args.reuse
    outs = dict()
    dp_i = 0
    while dp_i < len(design_points_tmp):
        dp = design_points_tmp[dp_i]
        title = titlefor(dp)
	if title not in donealready:
            if not title in outs:
                if args.reuse > 0:
                    if num_left > 0:
                        outs[title] = DictWriter(open("../results_refined/" + title, 'w'), exp.keys()+['Random', 'Prior', 'IPC', 'IO', 'Both', 'Count', 'Random-Time', 'Prior-Time', 'IPC-Time', 'IO-Time', 'Both-Time'])
                        outs[title].writeheader()
                        num_left -= 1
                else:
                    outs[title] = DictWriter(open("../results_refined/" + title, 'w'), exp.keys()+['Random', 'Prior', 'IPC', 'IO', 'Both', 'Count', 'Random-Time', 'Prior-Time', 'IPC-Time', 'IO-Time', 'Both-Time'])
                    outs[title].writeheader()
            if args.reuse > 0:
                if title in outs.keys():
                    design_points.append(dp)
            else:
	        design_points.append(dp)
        dp_i += 1

    if len(design_points) == 0:
        print "No design points"
        return 1

    print "Total design points: ", len(design_points)
    start = time.time()
    print "start:", start
    if args.sequential != 0:
	sequential_tests(outs, design_points)
    else:
        thread_tests(args, outs, design_points)
    end = time.time()
    print ""
    print "end:", end
    
    print "time:", end-start
    print "in hours:", (end-start)/3600.0

if __name__ == '__main__':
    main()
