from collections import defaultdict
from csv import DictReader
import numpy as np

oheadtypes =   {'CXS' : 'CXS', 
                'ISR' : 'ISR',
                'SCH' : 'SCHED_A',
                'SCL' : 'SCHED_C',
                'RLA' : 'RELEASE_LATENCY_A',
                'RLC' : 'RELEASE_LATENCY_C',
                'RQA' : 'RELEASE',
                'RQC' : 'RELEASE_C',
                'IPI' : 'SCHED_C',
                'TCK' : 'TICK',
                }
oheadcompl =   {'CXS' : 'monotonic', 
                'ISR' : 'linear',
                'SCH' : 'linear',
                'SCL' : 'linear',
                'RLA' : 'monotonic',
                'RLC' : 'monotonic',
                'RQA' : 'linear',
                'RQC' : 'linear',
                'IPI' : 'monotonic',
                'TCK' : 'monotonic',
                }

schedtypes = {'mc2', 'pedf'}

crittypes = {'Max', 'Avg'}


#---------------messsage queues------------------#

#in bytes
message_sizes 	= 	[64,		128,	256,	512,	1024,	2048,	4096,	8192]
#in us
sbp_send	=	[177.67,	182.0,	193.0,	199.0,	204.0,	240.0,	374.0,	538.0]
ce_send		=	[13.33,		13.33,	14.33,	15.0,	16.33,	17.0,	26.0,	35.33]
cl_send		=	[5.33,		5.00,	5.67,	5.67,	7.67,	13.0,	16.67,	27.0]
sbp_recv	=	[47.0,		48.67,	50.0,	53.67,	64.0,	76.67,	119.33,	257.33]
ce_recv		=	[12.0,		12.33,	13.33,	13.67,	14.33,	16.33,	22.0,	46.0]
cl_recv		=	[5.67,		5.67,	6.33,	6.33,	9.67,	10.0,	19.0,	27.33]
sbp_send_avg	=	[112.54,	113.17,	115.42,	119.89,	126.8,	142.99,	271.87,	393.55]
ce_send_avg	=	[11.33,		11.28,	11.99,	13.35,	15.89,	20.92,	33.54,	54.19]
cl_send_avg	=	[1.41,		1.44,	1.63,	1.85,	2.34,	3.32,	5.59,	10.34]
sbp_recv_avg	=	[21.04,		21.99,	24.19,	28.18,	35.73,	51.66,	92.02,	203.64]
ce_recv_avg	=	[9.05,		9.98,	11.14,	12.85,	16.37,	23.36,	39.11,	67.72]
cl_recv_avg	=	[1.72,		1.74,	1.89,	2.16,	2.68,	3.76,	6.21,	10.94]
ipcDict = dict( [('R', dict()), ('W', dict()) ])
ipcDictAvg = dict( [('R', dict()), ('W', dict()) ])

ipcDict['R']['SBP'] = dict(zip(message_sizes, sbp_recv))
ipcDict['W']['SBP'] = dict(zip(message_sizes, sbp_send))
ipcDict['R']['CE'] = dict(zip(message_sizes, ce_recv))
ipcDict['W']['CE'] = dict(zip(message_sizes, ce_send))
ipcDict['R']['CL'] = dict(zip(message_sizes, cl_recv))
ipcDict['W']['CL'] = dict(zip(message_sizes, cl_send))

ipcDictAvg['R']['SBP'] = dict(zip(message_sizes, sbp_recv_avg))
ipcDictAvg['W']['SBP'] = dict(zip(message_sizes, sbp_send_avg))
ipcDictAvg['R']['CE'] = dict(zip(message_sizes, ce_recv_avg))
ipcDictAvg['W']['CE'] = dict(zip(message_sizes, ce_send_avg))
ipcDictAvg['R']['CL'] = dict(zip(message_sizes, cl_recv_avg))
ipcDictAvg['W']['CL'] = dict(zip(message_sizes, cl_send_avg))
#----------------Buffer Access--------------------#

#measured using get_cycles which apparentally needs to be divided by 3 to get us
#Email Subject: "DMA interference data" Jan 29 2018
cam_copy_C = 5321.33/3
cam_copy_AB = 2925.67/3
cam_copy_C_avg = 4265.96/3
cam_copy_AB_avg = 2311.65/3

#-----------------DMA Interrupt penalties us-----------------#
cam_int = 0.065
cam_int_avg = 0.031

cam_usb_cpy = 2.460
cam_usb_cpy_avg = 0.5595

#---------------SSD---------------------------------------#

ssd_int = 0.164
ssd_int_avg = 0.0294

#-----------------Release IPI penalties-------------------#
ipi_int = 0.036
ipi_int_avg = 0.0217


# LINE_ACC_LOC = .523
LINE_ACC_CBR = 1.93
LINE_ACC_CBRavg = 1.64

LINE_ACC_CLCR = .131
LINE_ACC_CLCRavg = .069


LINE_ACC_CBW = 2.01
LINE_ACC_CBWavg = 1.64

LINE_ACC_CLCW = .18
LINE_ACC_CLCWavg = .073

CPMD = 40.0
CPMDavg = 21.0
PTCK = 10000.0

ohcvals = None
ohfuncs = None

class Ohead():
    def __init__(self, tss, data, complexity):
        self.complexity = complexity
        self.values = []
        maxval = -1
        points = 0
        for i in range(0, len(data)-1):
            maxval = max(maxval, data[i])
            self.values.append([])
            self.values[i].append(tss[i])
            self.values[i].append(maxval)
            
    def val(self, tss):
        if self.complexity is "monotonic":
            return self.valmono(tss)
        if self.complexity is "linear":
            return self.vallinear(tss)
        
    def valmono(self, tss):
        if self.values[0][0] >= tss:
            return self.values[0][1]
            
        if self.values[len(self.values)-1][0] < tss:
            return self.values[len(self.values)-1][1]
        for i in range(1, len(self.values)):
            if(self.values[i][0] >= tss):
                p1 = self.values[i-1]
                p2 = self.values[i]
                bary = float((tss-p1[0]))/float(p2[0]-p1[0])
                return p1[1]+bary*(p2[1]-p1[1])
    
    def vallinear(self, tss):
        if self.values[0][0] >= tss:
            return self.values[0][1]
        
        p1 = None
        p2 = None
        if self.values[len(self.values)-1][0] < tss:
            p1 = self.values[len(self.values)/2]
            p2 = self.values[len(self.values)-1]
        else:
            for i in range(1, len(self.values)):
                if(self.values[i][0] >= tss):
                    p1 = self.values[i-1]
                    p2 = self.values[i]
                    break
                    
        bary = float((tss-p1[0]))/float(p2[0]-p1[0])
        return p1[1]+bary*(p2[1]-p1[1])
                
def SetOheads(mcx_ts, scheduler):
    global ohcvals
    
    tss = len(mcx_ts)
    ohcvals = defaultdict()
    for crit in crittypes:
        ohcvals[crit] = defaultdict()
        for ohd in oheadtypes:
            ohcvals[crit][ohd] = ohfuncs[scheduler][crit][ohd].val(tss)
            if scheduler is 'pedf':
                ohcvals[crit][ohd] *= mcx_ts.a_inflate_choice
