#!/usr/bin/env python

import sqlite3
import os
import sys
import csv 

dirname = ''
if len(sys.argv) == 2:
	dirname = sys.argv[1]
else:
	for dbfile in os.listdir("."):
		if dbfile.endswith('.db'):
			dirname = dbfile[:-3]
	if dirname is '':
		print 'provide a results directory'
		sys.exit()


conn = None
conn = sqlite3.connect(dirname + '.db')

def apeq(f1,f2,epsilon=1e-3):
	return abs(f1-f2) < epsilon

def dict_factory(curse, row):
	d = {}
	for idx, col in enumerate(curse.description):
		d[col[0]] = row[idx]
	return d

conn.row_factory = dict_factory

curses = conn.cursor()

def calcSUA(whereStr=""):
	if whereStr is not "":
		whereStr = " WHERE " + whereStr
	curses.execute("SELECT AVG(random) AS random, AVG(prior) AS prior, AVG(io) AS io, AVG(ipc) AS ipc, AVG(both) AS both, COUNT(*) as num FROM sua_scen" + whereStr)
	data = curses.fetchone()
	return data	

def getWinnerCount(whereStr=""):
	if whereStr is not "":
		whereStr = " WHERE " + whereStr
	curses.execute("SELECT random, prior, io, ipc, both FROM sua_scen" + whereStr)
	data = curses.fetchall()
	winnerDict = dict( [ (sched, 0) for sched in ['uedf','prior','io','ipc','both'] ] )
	for row in data:
		invDict = dict()
		for key in row.keys():
			invDict[row[key]] = key
		winVal = max(row.values())
		winnerDict[invDict[winVal]]+=1
	return winnerDict

def calcPerDiff(v1, v2):
	return 2*(v2-v1)/(v2+v1)

#def calcPerNoIO(whereStr=""):
#	if whereStr is not "":
#		whereStr = " WHERE " + whereStr
#	curses.execute("SELECT AVG(scen.uedf/nio.os_iso) AS uedf, \
#		AVG(scen.os_iso/nio.os_iso) AS os_iso, \
#		AVG(scen.prior/nio.os_iso) AS prior FROM sua_scen AS scen, sua_noio AS nio \
#		WHERE scen.scenario IS nio.scenario")
#	data = curses.fetchone()
#	return data 

"""
dists = {
	'util_dist':['util_Amod_Bmod_Clight','util_Alight_Blight_Cheavy','util_Amod_Bmod_Cmod'],
	'period':['period_medium','period_realistic','period_long'],
	'task_util':['lightweight','modweight','heavyweight'],
	'memory_util':['wss-light','wss-heavy'],
	'ipc_size':['message_large','message_small'],
	'bandwidth':['band_low','band_mod','band_high'],
	'copy_dist':['copy_heavy','copy_light'],
}
"""
#dists = {
#	'util_dist':['util_Amod_Bmod_Clight','util_Alight_Blight_Cheavy'],
#	'period':['period_short','period_long'],
#	'task_util':['lightweight','heavyweight'],
#	'memory_util':['wss-heavy'],
#	'pet_sens':['sens_regular'],
#	'net_dist':['net_none','net_few','net_many'],
#	'has_cam':['has_cam','no_cam']
#}


"""
Motivating Observations:
1. OS-ISO vs Prior
2. Network vs Camera
3. OS-ISO vs Non-IO
"""

#reqs = [
#	"",
	#"NOT (net_dist IS 'net_none' AND has_cam IS 'no_cam')",
	#"net_dist IS NOT 'net_none' AND has_cam IS 'no_cam'",
	#"net_dist IS 'net_none' AND has_cam IS NOT 'no_cam'",
#	"dma_active IS 'dma_short'",
#	"dma_active IS 'dma_short' AND io_size IS 'io_large'",
#	"dma_active IS 'dma_short' AND io_size IS 'io_small'",	
#]

#for req in reqs:
#	print req
#	sua = calcSUA(req)
	#pd = calcPerDiff(sua['prior'], sua['os_iso'])
#	print sua
	#print pd
#	print '---------------------------------'


#fracNOIO = calcPerNoIO()
#print fracNOIO

noCond = calcSUA("")
messLarge = calcSUA("ipc_size IS 'message_large'")
bandHigh = calcSUA("bandwidth IS 'band_high'")
bandLow = calcSUA("bandwidth IS 'band_low'")
bandMed = calcSUA("bandwidth IS 'band_mod'")
cpHeavy = calcSUA("copy_dist IS 'copy_heavy'")

print "General:", noCond
print "Copy Heavy:", cpHeavy

print "P(Prior)/P(Random):", noCond['prior']/noCond['random']
print "P(IPC)/P(Prior):", noCond['ipc']/noCond['prior']
print "P(IPC|Mess Large)/P(Prior|Mess Large)", messLarge['ipc']/messLarge['prior']
print "P(IO)/P(Prior):", noCond['io']/noCond['prior']
print "P(IO|Low)/P(Prior|Low):", bandLow['io']/bandLow['prior']
print "P(IO|Med)/P(Prior|Med):", bandMed['io']/bandMed['prior']
print "P(IO|High)/P(Prior|High):", bandHigh['io']/bandHigh['prior']
print "P(BOTH)/P(Prior):", noCond['both']/noCond['prior']
print "P(BOTH)/P(Random):", noCond['both']/noCond['random']

conn.close()

