#!/usr/bin/env python

import sqlite3
import os
import sys
import csv 
import numpy as np

dirname = ''
if len(sys.argv) == 2:
	dirname = sys.argv[1]
else:
	for dbfile in os.listdir("."):
		if dbfile.endswith('.db'):
			dirname = dbfile[:-3]
	if dirname is '':
		print 'provide a results directory'
		sys.exit()

conf_dir = None
if 'CONF_DIR' in os.environ:
	conf_dir = os.environ['CONF_DIR']
else:
	print 'Conference Directory not set'
	sys.exit()



#--------------------------------Headers--------------------------------------#
distributions = [
	'util_dist',	'period',	'task_util', 
	'memory_util',	'ipc_size',	'copy_dist',
	'bandwidth'
]

scheds = [
	'random', 'prior', 'io', 'ipc', 'both' 
]

times = [
	'Random-Time', 'Prior-Time', 'IO-Time', 'IPC-Time', 'Both-Time'
]

cols = distributions + scheds + ['sys_util'] + ['count']
types = ['TEXT' for i in distributions] + ['REAL' for i in scheds] + ['REAL'] + ['INTEGER']

#-------------------------------Open DB connection------------------------------#
conn = None
conn = sqlite3.connect(dirname + '.db')
curses = conn.cursor()
curses.execute('DROP TABLE IF EXISTS raw')

curses.execute('CREATE TABLE raw(' + ', '.join([' '.join(colANDtype) for colANDtype in zip(cols,types)]) + ')')


#------------------------------Read in raw--------------------------------------#

#create mapping from csv file colnames to this file's colnames
dis_dict = [(dis,dis) for dis in distributions]
sched_dict = [('random','Random'),('prior','Prior'),('io','IO'),('ipc','IPC'),('both','Both')]
mapping = dict(dis_dict + sched_dict + [('sys_util','sys_util'), ('count','Count')])

### Table raw is in the same format as raw data files, except columns not in distributions or sched are dropped ###	
ins_str = "INSERT INTO raw VALUES(" + ','.join(['?' for i in range(len(cols))]) + ")"
fullpath = conf_dir + "/" + dirname
totTime = 0
counts = dict( [ (i, 0) for i in range(100, 2001)] )
for res in os.listdir(fullpath):
	with open(fullpath + "/" + res, 'rb') as resfile:
		#print res
		cr = csv.DictReader(resfile)
		vals = [] #in order of table
		for row in cr: #in order of file
			for schedTimes in times:
				#print schedTimes
				if row[schedTimes] is not None and row['Count'] is not None:
					totTime += float(row[schedTimes])*int(row['Count'])
                                        counts[int(row['Count'])] += 1
			val = [None for i in range(len(cols))]
			for i in range(len(cols)):
				col = cols[i]
				if col not in distributions and row[mapping[col]] is not '':
					val[i] = float(row[mapping[col]])
				else:
					val[i] = row[mapping[col]]	
			assert all([ val[i] is not None for i in range(len(cols)) ])
			vals.append(val)
		curses.executemany(ins_str, vals) 
		conn.commit()

for key in counts:
    print key,"\t",counts[key]
print totTime/60/60/24

#----------------------------Compute SUAs from raw-----------------------------#

###padsum and padcase implement trap rule###
padsum = lambda instr: 'sum(' + instr + ') AS ' + instr
padcase = lambda instr: 'CASE round(sys_util, 1) WHEN 0.2 THEN 0.5*' + instr + ' WHEN 9.0 THEN 0.5*' + instr + ' ELSE ' + instr + ' END AS ' + instr

curses.execute( 'DROP TABLE IF EXISTS suatmp' )
curses.execute( 'DROP TABLE IF EXISTS sua' )

select_str = 'SELECT ' + ','.join(distributions + map(padsum, scheds)) + ' FROM suatmp '
select2_str = 'SELECT ' + ','.join(distributions + map(padcase, scheds)) + ' FROM raw'
group_str = 'GROUP BY ' + ','.join(distributions)

curses.execute( 'CREATE TABLE suatmp AS ' + select2_str)
curses.execute( 'CREATE TABLE sua AS ' + select_str + group_str)
curses.execute( 'DROP TABLE suatmp')

curses.execute( "ALTER TABLE sua ADD COLUMN scenario TEXT" )
curses.execute( "UPDATE sua SET scenario=util_dist || period || task_util || memory_util || ipc_size || copy_dist || bandwidth" )

curses.execute( "DROP TABLE IF EXISTS sua_scen" )
curses.execute( "CREATE TABLE sua_scen AS SELECT * FROM sua WHERE prior IS NOT NULL" )

curses.execute( "DROP TABLE sua" )

curses.execute( "ALTER TABLE raw ADD COLUMN scenario TEXT" )
curses.execute( "UPDATE raw SET scenario=util_dist||period||task_util||memory_util||ipc_size||copy_dist||bandwidth||ROUND(sys_util,1)" )

conn.commit()

conn.close()
